`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: BASTLI
// Engineer: Lukas Schrittwieser
// 
// Create Date:    19:37:00 02/27/2009 
// Design Name: 
// Module Name:    main 
// Project Name:   CPLD demo project 3 - audio
// Target Devices: xc9536xl / xc9572xl
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module main(
    input CLK_FAST,	// use 256kHz clock
	input CLK_SLOW, // use 250Hz clock
    output [1:0] LED,
    //output LED_1,
    output [3:0] AUDIO,
    input TASTER_RES
    );
	
	reg [2:0] sine_tbl_ind;
	reg [3:0] audio_out;	// dummy signal for audio output
	reg sine_ind_up;
	reg rst = 1'b1;
	reg sine_cnt_en;
	reg [5:0] note_cnt;
	reg [2:0] note;
	
	reg [6:0] song_delay;	// delay counter to control the speed of the song
	//reg	next_note;			// the song index increments if this is one
	reg [5:0] song_ind;		// index for the song storage
	
	// bind IOs to internal signals
	assign AUDIO = audio_out;
	//assign rst <= ! TASTER_R; 

	always @ (posedge CLK_FAST)
	begin
		rst <= ~TASTER_RES;
	end	

	assign LED[0] = song_ind[0];
	assign LED[1] = ~sine_ind_up;

	always @ (posedge CLK_SLOW)
	begin
		if (rst == 1'b1)
		begin
			song_delay 	<= 7'h00;
			//next_note	<= 1'b0;
			song_ind 	<= 6'h00;
		end else
		begin
			// increment the counter
			song_delay <= song_delay + 1;
			// if the counter reaches zero we set next_note high for one period
			if (song_delay == 7'h00)
			begin
				song_ind	<= song_ind + 1;
				// if the counter reaches zero we go to the next note
				if (song_ind == 39) // we have 40 notes in our song
					song_ind <= 0;
				//next_note <= 1'b1;
			end	
			//else
			//	next_note <= 1'b0;
				
			// the following line defines the speed of the song	
			if (song_delay == 7'd110)
				song_delay <= 0;
			else
				song_delay <= song_delay + 1;
		end
	end

	// song index counter - generates the index for the note storage
//	always @ (posedge CLK_SLOW)
//	begin
//		if (rst == 1'b1)
//		begin
//			song_ind 	<= 6'h00;
//		end else
//		begin
//			// increment the counter
//			if (next_note == 1'b1)
//			begin
//				song_ind	<= song_ind + 1;
//				// if the counter reaches zero we set next_note high for one period
//				if (song_ind == 39) // we have 40 notes in our song
//					song_ind <= 0;
//			end		
//		end
//	end

	// note memory - it stores the notes for our song
	always @ (song_ind)
	begin
		case (song_ind)
			6'd00 : note <= 3'h1;
			6'd01 : note <= 3'h2;
			6'd02 : note <= 3'h3;
			6'd03 : note <= 3'h4;
			6'd04 : note <= 3'h5;
			6'd05 : note <= 3'h5;
			6'd06 : note <= 3'h5;
			6'd07 : note <= 3'h5;
			6'd08 : note <= 3'h6;
			6'd09 : note <= 3'h6;
			6'd10 : note <= 3'h6;
			6'd11 : note <= 3'h6;
			6'd12 : note <= 3'h5;
			6'd13 : note <= 3'h5;
			6'd14 : note <= 3'h0;
			6'd15 : note <= 3'h0;
			6'd16 : note <= 3'h6;
			6'd17 : note <= 3'h6;
			6'd18 : note <= 3'h6;
			6'd19 : note <= 3'h6;
			6'd20 : note <= 3'h5;
			6'd21 : note <= 3'h5;
			6'd22 : note <= 3'h0;
			6'd23 : note <= 3'h0;
			6'd24 : note <= 3'h4;
			6'd25 : note <= 3'h4;
			6'd26 : note <= 3'h4;
			6'd27 : note <= 3'h4;
			6'd28 : note <= 3'h3;
			6'd29 : note <= 3'h3;
			6'd30 : note <= 3'h3;
			6'd31 : note <= 3'h3;
			6'd32 : note <= 3'h5;
			6'd33 : note <= 3'h5;
			6'd34 : note <= 3'h5;
			6'd35 : note <= 3'h5;
			6'd36 : note <= 3'h1;
			6'd37 : note <= 3'h1;
			6'd38 : note <= 3'h0;
			6'd39 : note <= 3'h0;
			default: note <= 3'bXXX;
		endcase
	end

	// note generator
	// translates a 3 bit note value into an sine counter enable strobe
	// note encodes as:
	//  0: silence
	//  1: C'' (522Hz)
	//  2: D'' (590Hz)
	//  3: E'' (653Hz)
	always @ (posedge CLK_FAST)
	begin
		if (rst == 1'b1)
		begin
			sine_cnt_en <= 1'b0;
			note_cnt <= 6'h00;
		end else
		begin
			// if the counter wraped to zero we indecate an increment for the sine table index counter
			if ((note_cnt == 6'h0) && (note != 3'h0)) 
				sine_cnt_en <= 1'b1;
			else
				sine_cnt_en <= 1'b0;
			
			// check the maximum value according to the desired note
			if (note == 3'h1) begin
				if (note_cnt == 34)
					note_cnt <= 0;
				else
					note_cnt <= note_cnt + 1;	// increment the counter
			end else if (note == 3'h2) begin
				if (note_cnt >= 6'd30)
					note_cnt <= 0;
				else
					note_cnt <= note_cnt + 1;	// increment the counter
			end else if (note == 3'h3) begin
				if (note_cnt >= 6'd27)
					note_cnt <= 0;
				else
					note_cnt <= note_cnt + 1;	// increment the counter
			end else if (note == 3'h4) begin
				if (note_cnt >= 6'd25)
					note_cnt <= 0;
				else
					note_cnt <= note_cnt + 1;	// increment the counter
			end else if (note == 3'h5) begin
				if (note_cnt >= 6'd22)
					note_cnt <= 0;
				else
					note_cnt <= note_cnt + 1;	// increment the counter
			end else if (note == 3'h6) begin
				if (note_cnt >= 6'd20)
					note_cnt <= 0;
				else
					note_cnt <= note_cnt + 1;	// increment the counter
			end else if (note == 3'h7) begin
				if (note_cnt >= 6'd17)
					note_cnt <= 0;
				else
					note_cnt <= note_cnt + 1;	// increment the counter
			end
		end	
	end

	

	// table index counter
	// this counter together with the cosine table form an sine wave generator
	always @ (posedge CLK_FAST)
	begin 
		
		if (rst == 1'b0)
		begin
			if (sine_cnt_en == 1'b1)
			begin	
				if (sine_ind_up == 1'b1)
					sine_tbl_ind <= sine_tbl_ind + 1;
				else
					sine_tbl_ind <= sine_tbl_ind - 1;
				if (sine_tbl_ind == 3'b110)
					sine_ind_up <= 1'b0;
				if (sine_tbl_ind == 3'b001)
					sine_ind_up <= 1'b1;
			end 
		end	
		else begin
			sine_ind_up <= 1'b1;
			sine_tbl_ind <= 3'b000;
		end
	end

	// cosine half wave table 
	// 4 bit sine_tbl_ind: audio_out = 7.5*(1-cos(sine_tbl_ind*pi/15))
	// 3 bit sine_tbl_ind: audio_out = 7.5*(1-cos(sine_tbl_ind*pi/7))
	always @ (sine_tbl_ind)
	begin
		case (sine_tbl_ind)
//			4'h0 : audio_out = 4'h0;
//			4'h1 : audio_out = 4'h0;
//			4'h2 : audio_out = 4'h1;
//			4'h3 : audio_out = 4'h1;
//			4'h4 : audio_out = 4'h2;
//			4'h5 : audio_out = 4'h4;
//			4'h6 : audio_out = 4'h5;
//			4'h7 : audio_out = 4'h7;
//			4'h8 : audio_out = 4'h8;
//			4'h9 : audio_out = 4'ha;
//			4'ha : audio_out = 4'hb;
//			4'hb : audio_out = 4'hd;
//			4'hc : audio_out = 4'he;
//			4'hd : audio_out = 4'he;
//			4'he : audio_out = 4'hf;
//			4'hf : audio_out = 4'hf;
			
			3'h0 : audio_out = 4'h0;
			3'h1 : audio_out = 4'h1;
			3'h2 : audio_out = 4'h3;
			3'h3 : audio_out = 4'h6;
			3'h4 : audio_out = 4'h9;
			3'h5 : audio_out = 4'hc;
			3'h6 : audio_out = 4'he;
			3'h7 : audio_out = 4'hf;
			
		endcase
	end	
		
endmodule
